/*
 * Client to the IVOP service. This is a Symphony Asynchronous client
 */ 

#include"stdafx.h"
#include"soam.h"
#include"MyMessage.h"
#include"MyCallback.h"
#include"unistd.h"
#include<string.h>

using namespace soam;
using namespace std;

// my tools are in C in the libtools.a library 
extern "C" {
    extern char     *getinput(const char *);
    extern double   c2d(const char *, int *);
    static char     *user_input(int *);
    static int      load_math_params(char *);
    extern char     *nextline_(FILE *fp);
}


// a simple cross-platform wrapper for sleep(...)
void ourSleep(unsigned long sleepInSeconds)
{
#ifdef WIN32
    soam::Sleep(sleepInSeconds * 1000);
#else
    sleep(sleepInSeconds);
#endif
}

int 
main(int argc, char** argv)
{
    try {

        // initialize the API
        SoamFactory::initialize();

        // setup application specific information to be 
	// supplied to the System
        const char appName[] = "ivop";

        // setup application authentication information using the 
	// default security provider,
        DefaultSecurityCallback securityCB("Guest", "Guest");

        // connect to the specified application
        ConnectionPtr conPtr = SoamFactory::connect(appName, &securityCB);

        // retrieve and print our connection ID
        cout << "connection ID=" << conPtr->getId() << endl; 

        // create call back
        MySessionCallback  myCallback;

        // Create an asynchronous Session
	// No common data object required in this example
        SessionPtr sesPtr = conPtr->createSession("mySession",
						  "ShortRunningTasks",
						  SF_RECEIVE_ASYNC,
						  SOAM_NULL_PTR, 
						  &myCallback);
	
        // retrieve and print session ID
        cout << "Session ID:" << sesPtr->getId() << endl;

	// Create and initialize the protocol message buffer 
	// between me and my computational service.
	char   protocol_message[1024];
	memset(protocol_message, 0, sizeof(protocol_message));

	// Load the mathematical parameters
	// of the Heston model
	load_math_params(protocol_message);
	int L = strlen(protocol_message);

	int tasks = 0;
	while (1) {

	    int   err;
	    char *uinput = user_input(&err);
	    if (uinput == NULL) {
		if (err)
		    cerr << "Error in user input" << endl;
		break;
	    }

	    sprintf(protocol_message + L, " %s",  uinput);
	    // Create a message
	    ++tasks;
	    MyMessage inMsg(1, false, protocol_message);

	    cout << "Computing" << flush;
	    // send it
	    TaskInputHandlePtr input = sesPtr->sendTaskInput(&inMsg);
	    
	    // retrieve and print task ID
	    // cout << "task submitted with ID : " << input->getId() << endl;

	    while ((myCallback.getReceived() < tasks) 
		   && !myCallback.getDone()) {
		cout << "." << flush ;
		ourSleep(1);
	    }
	    cout << "done" << endl;
	} /* while (uinput) */

    } /* try */

    catch(SoamException& exp) {
	// report exception
	cout << "exception caught ... " << exp.what() << endl;
    }
    
    // uninitialize the API
    // This is the only means to ensure proper shutdown 
    // of the interaction between the client and the system.
    SoamFactory::uninitialize();
    
    return(0);

} // main() 


static char *
user_input(int *err)
{
    static char   *v[4] = 
	{"v0 = ", "eprice = ", "tau = ", "rtax = "};
    static char   buf[1024];

    memset(buf, 0, sizeof(buf));

    for (unsigned int n = 0; 
	 n < sizeof(v)/sizeof(v[0]); 
	 n++) {
	char   *p;
	double z;

	p = getinput(v[n]);
	if (p == NULL || 
	    strcmp(p, "") == 0) {
	    *err = 0;
	    return(NULL);
	}

	z = c2d(p, err);
	if (z < 0 && *err == 1)
	    return(NULL);

	sprintf(buf + strlen(buf), "%s ", p);

    } // for
    
    return(buf);
    
} // user_input() 

// Load the mathematical parameters of the Heston model.
// dstep eps chi theta rho 
static int
load_math_params(char *buf)
{
    char    dbuf[64];
    FILE    *fp;
    char    *p;
    float   dstep;
    float   eps;
    float   chi;
    float   theta;
    float   rho;
    int     cc;

    //prepare the default buffer
    sprintf(dbuf, "0.0 0.0 0.0 0.0 0.0 ");
    strcpy(buf, dbuf);

    // first try to open the file 
    // if it exists
    fp = fopen("heston.params", "r");
    if (fp == NULL)
	return(-1);
    
    // get the first non comment line from
    // the file
    p = nextline_(fp);
    fclose(fp);
    if (p == NULL) 
	return(-1);

    //verify
    cc = sscanf(p, "\
%f%f%f%f%f", &dstep, &eps, &chi, &theta, &rho);
    if (cc != 5)
	return(-1);

    //ok return the math parameters
    // by overwriting the default values...
    sprintf(buf, "%s ", p);

    // ciao
    return(0);

} // load_math_params() 
