c
c This program is a subroutine that computes the price of an 
c integrated variance option when the asset dynamics is given 
c by the Heston model
c

      integer function heston1dim(dstep, eps, chi, theta, rho, v0, 
     & eprice, tau, rtax, option)
      real*8   dstep, eps, chi, theta, rho, v0
      real*8   eprice, tau, rtax
      integer n1
      parameter(n1 = 2048*2048)
      complex*16   ff(0:n1), xi, contrib
      complex*16   ui, beta, zita, gamma, csum
      real*8       xiv(0:n1), nris
      real*8       vero, s, kchi, pi, pi2
      real*8       x0, ss, i0, option
      real*4       timep, secnds
      integer      j, mm
      external     secnds 

      pi = dacos(-1.d0)

      pi2 = 2.d0*pi
      ui = dcmplx(0.d0,1.d0)

c The oscillatory integral (30) is computed using the method proposed 
c by Bailey, Swarztrauber in the paper published on Siam Journal 
c on Scientific Computing Vol 15(5) 1994 p. 1105-1110


c dstep: real number, generally a power of two that must be assigned
c to determine the grid  of integration. 
c Hint: dstep=256 or 512 (dstep<=2048)


      if (dstep .eq. 0.d0) then
         dstep = 256.d0
      endif

      nris = dsqrt(pi2)/dstep
      mm = int(pi2/nris**2)

c  Definition of the integration grid

      do j = 0, mm-1
         xiv(j) = dfloat(j-mm/2)*nris
      enddo

c Assign the parameters of the Heston model
c eps= parameter epsilon appearing in formulae (5) and (14)
c chi= parameter chi appearing in formuale (5) and (14)
c theta= parameter theta appearing in formuale (5) and (14)
c rho
c Parameters for the experiment

      if (eps .eq. 0.d0) then
         eps = 0.2d0
      endif

      if (chi .eq. 0.d0) then
         chi = 0.8d0
      endif

      if (theta .eq. 0.d0) then
         theta = eps**2/(2.d0*chi)+0.1d0
      endif

      if (rho .eq. 0.d0) then
         rho = 0.1d0
      endif

c   i0: initial integrated variance, i0=0 equation (10)
c   x0: initial centrated log-returns x0=0 equation (16)

      i0 = 0.0d0
      x0 = 0.d0

c Assign: v0, eprice, tau, rtax

c     v0: initial volatility equation (17) 
c     eprice: integrated variance strike price (that is the strike
c             price of the option on the realized variance) 
c     tau: time to maturity 
c     rtax: risk free interest rate 

      if (v0 .eq. 0.d0) then
         v0 = 2.0d0
      endif

      if (eprice .eq. 0.d0) then
         eprice = 0.5d0
      endif

      if (tau .eq. 0.d0) then
         tau = 1.d0
      endif
      
      if (rtax .eq. 0.d0) then
         rtax = 0.d0
      endif

      s = 2.d0*chi*theta/eps**2-1.d0

c Note that s must be greater than zero
      
      ss = s + 1
      kchi = chi - eps*0.5d0*rho
      
c Start integration procedure

      timep = secnds(0.0)

      do j = 0, mm-1
         xi = xiv(j) 
         zita = 0.5d0 * cdsqrt(chi**2 + 2.d0*eps**2*xi*ui)
         beta = chi*0.5 + zita + (zita-0.5*chi)*cdexp(-2.d0*tau*zita)
         gamma = 1.d0-cdexp(-2.d0*tau*zita)
         ff(j) = cdexp(-ss*(-0.5*chi+zita)*tau+ss*cdlog(2.d0*zita/beta)
     &        - v0*ui*xi*gamma/beta)

         if (cdabs(xi) .gt. 1.d-6) then
            contrib = (-eprice)/(ui*xi)
            contrib = contrib+(cdexp(ui*xi*eprice)-1.d0)
     &           /     ((ui*xi)*(ui*xi))
         else
            contrib = (eprice)*(eprice)*0.5d0
         endif
         ff(j) = ff(j)*contrib 
      enddo

      csum = (0.d0,0.d0)

      do j = 0,mm-1
         csum = csum+(-1.d0)**j*ff(j)
     &        *cdexp(-2.d0*pi*dfloat(mm*j)*0.5*ui/dfloat(mm))
      enddo

      csum = csum*(-1.d0)**(mm*0.5)*nris/pi2
      vero = i0-eprice+theta*tau+(1.d0-dexp(-chi*tau))*(v0-theta)/chi
      csum = vero+csum

c Compute the discounted value

      option = dexp(-rtax*tau)*dreal(csum)

c Compute the execution time of this program
      timep = secnds(timep)

      open(4, file = 'heston1dim.dat', status = 'unknown')
      write(4,*) 'time = ', timep
      write(4,*) 'option value = ', option
      write(4,*) 'imaginary part ', dimag(csum)   

      close(4)

      return

      end
